<?php

/**
 *
 * $Id: class.AbstractUserDAO.php,v 1.13 2005/05/26 21:55:48 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     dao
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.13 $
 *
 */

/**
 * coWiki - Abstract user DAO class
 *
 * @package     dao
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.3.0
 *
 * @todo        [D11N]  Complete documentation
 */
abstract class AbstractUserDAO extends PersistentObservable {

    protected
        $aIgnLogin   = array(),
        $aIgnUid     = array(),
        $aIgnGrpName = array(),
        $aIgnGid     = array(),

        $Users = null,
        $Groups = null;

    // --------------------------------------------------------------------

    /**
     * Class constructor
     *
     * @access  protected
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    protected function __construct() {
        $Context = RuntimeContext::getInstance();
        $Registry = $Context->getRegistry();

        // Initialize ignored account/groups names and ids
        $sLogin = $Registry->get('.USER_IGNORE_LOGIN');
        $sId    = $Registry->get('.USER_IGNORE_UID');
        $this->aIgnLogin = explode(',', $sLogin);
        $this->aIgnUid   = explode(',', $sId);

        // Initialize ignored account/groups names and ids
        $sName = $Registry->get('.USER_IGNORE_GROUP_NAME');
        $sId   = $Registry->get('.USER_IGNORE_GID');
        $this->aIgnGrpName = explode(',', $sName);
        $this->aIgnGid     = explode(',', $sId);

        // ----------------------------------------------------------------

        $this->Groups = new Vector;
        $this->Users = new Vector;

        // Set default groups
        $Group = new Group();
        $Group->set('groupId', 0);
        $Group->set('name', 'wheel');
        $this->Groups->add($Group);

        // ---

        $Group = new Group();
        $Group->set('groupId', 65535);
        $Group->set('name', 'guests');
        $this->Groups->add($Group);

        // ---

        // Set default users
        $User = new User();
        $User->set('userId', 0);
        $User->set('groupId', 0);
        $User->set('login', 'root');
        $User->set('name', __('I18N_ROOT_FULLNAME'));
        $User->set('isActive', true);
        $User->set('isLocked', false);

        $this->Users->add($User);

        // ---

        $User = new User();
        $User->set('userId', 65535);
        $User->set('groupId', 65535);
        $User->set('login', 'guest');
        $User->set('name', __('I18N_GUEST_FULLNAME'));
        $User->set('isActive', true);
        $User->set('isLocked', false);

        $this->Users->add($User);
    }

    // --------------------------------------------------------------------

    /**
     * Users stored in a SQL database has a "user active" flag, reflect it.
     * Abstract method to be implemented by a subclass.
     *
     * @access  public
     * @return  boolean Returns true if user storage has an active flag,
     *                  false otherwise.
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    abstract function hasUserActiveFlag();

    // --------------------------------------------------------------------

    /**
     * Users stored in a SQL database has a "user locked" flag, reflect it.
     * Abstract method to be implemented by a subclass.
     *
     * @access  public
     * @return  boolean Returns true if user storage has a locked flag,
     *                  false otherwise.
     *
     * @access  public
     * @param   integer
     * @param   string
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    abstract function hasUserLockedFlag();

    // --------------------------------------------------------------------

    /**
     * Get user by uid (user id). Returns null if no user has been found.
     *
     * @access  public
     * @param   integer User id
     * @return  object  User object or null if no user with this user id
     *                  has been found.
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    abstract function getUserByUid($nId);

    // --------------------------------------------------------------------

    /**
     * Get user by login. Returns null if no user has been found.
     *
     * @access  public
     * @param   integer User id
     * @return  object  User object or null if no user with this user id
     *                  has been found.
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    abstract function getUserByLogin($sLogin);

    // --------------------------------------------------------------------

    /**
     * Get all users.
     *
     * @access  public
     * @param   boolean
     * @return  Vector  Vector with user objects
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    abstract function getAllUsers($bGetGuests = true);

    // --------------------------------------------------------------------

    /**
     * Store user.
     *
     * @access  public
     * @param   object  User object
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    abstract function storeUser($User);

    // --------------------------------------------------------------------

    /**
     * Store users.
     *
     * @access  public
     * @param   object  Collection (Vector) of User objects
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$nUid"
     */
    abstract function storeUsers($Collection);

    // --------------------------------------------------------------------

    /**
     * Remove user.
     *
     * @access  public
     * @param   object  User object
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    abstract function removeUser($User);

    // --------------------------------------------------------------------

    /**
     * Get group by gid (group id). Returns null if no group with this id
     * has been found.
     *
     * @access  public
     * @param   integer Group id
     * @return  object  Group object or null if no group with this group id
     *                  has been found.
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    abstract function getGroupByGid($nId);

    // --------------------------------------------------------------------

    /**
     * Get all user groups.
     *
     * @access  public
     * @return  object  Vector containing Group objects
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    abstract function getAllGroups();

    // --------------------------------------------------------------------

    /**
     * Store user group.
     *
     * @access  public
     * @param   object  Group object.
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    abstract function storeGroup($Group);

    // --------------------------------------------------------------------

    /**
     * Store user groups.
     *
     * @access  public
     * @param   object  Collection (Vector) containing Group objects.
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    abstract function storeGroups($Collection);

    // --------------------------------------------------------------------

    /**
     * Remove user group.
     *
     * @access  public
     * @param   object  Group object.
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    abstract function removeGroup($Group);

    // --------------------------------------------------------------------

    /**
     * Return member groups of the given uid (user id)
     *
     * @access  public
     * @param   integer User id (uid)
     * @return  object  Vector containing users member Group objects
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     */
    abstract function getMemberGroupsOfUid($nId);

    // --------------------------------------------------------------------

    /**
     * Check if given user id and/or user login has to be ignored
     *
     * @access  public
     * @param   integer
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check the parameter type of "$nUid"
     */
    public function isIgnoredAccount($nUid, $sLogin) {
        if (in_array($nUid, $this->aIgnUid)) {
            return true;
        }

        for ($i=0, $n=sizeof($this->aIgnLogin); $i<$n; $i++) {
            if (preg_match(','.trim($this->aIgnLogin[$i]).',iS', $sLogin)) {
                return true;
            }
        }

        return false;
    }

    // --------------------------------------------------------------------

    /**
     * Check if given group id and/or group name has to be ignored
     *
     * @access  public
     * @param   integer
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check the parameter type of "$nGid"
     */
    public function isIgnoredGroup($nGid, $sName) {
        if (in_array($nGid, $this->aIgnGid)) {
            return true;
        }

        for ($i=0, $n=sizeof($this->aIgnGrpName); $i<$n; $i++) {
            if (preg_match(','.trim($this->aIgnGrpName[$i]).',iS', $sName)) {
                return true;
            }
        }

        return false;
    }

} // of class

?>
